# Python Environment (python_env)

**Python Environment** is a powerful developer tool that enables seamless integration between Drupal and Python. It provides a clean service-based architecture for executing Python scripts directly from PHP code within your Drupal site. This module is ideal for projects requiring AI/ML, data processing, automation, or custom logic handled in Python.

## Features

- Run Python scripts from Drupal using a simple service.
- Automatically creates a `/python/` folder at the root of your Drupal project.
- Write Python scripts that accept structured input and return structured output (JSON).
- Clean separation of logic between PHP and Python layers.
- Ideal for ML models, API clients, and complex data processing routines.

---

## Installation

1. Install the module using Composer:
   ```bash
   ddev composer require drupal/python_env
   ```

2. Enable the module via Drush or Drupal admin UI:
   ```bash
   ddev drush en python_env
   ```

3. On installation, a `/python/` directory will be created at the root of your Drupal project if it doesn't already exist. This is where your Python scripts should live.

---

## Usage

### 1. Add your Python scripts
Place `.py` files inside the `/python/` folder. Each script should:
- Accept a single JSON string as its argument.
- Return a JSON string as output.

**Example Python script: `/python/my_script.py`**
```python
import sys
import json

def run(args):
    name = args.get("name", "World")
    return {"greeting": f"Hello, {name}!"}

if __name__ == '__main__':
    args = json.loads(sys.argv[1])
    result = run(args)
    print(json.dumps(result))
```

### 2. Call from PHP
Use the provided `python_env.bridge` service to run the script:
```php
/** @var \Drupal\python_env\Service\PythonBridge $bridge */
$bridge = \Drupal::service('python_env.bridge');
$result = $bridge->run('my_script', ['name' => 'Alice']);
```

The output `$result` will be:
```php
['greeting' => 'Hello, Alice!']
```

---

## Error Handling
- If the script is missing or crashes, a PHP exception is thrown.
- If the script returns invalid JSON, an exception will also be raised.
- All scripts are run via Symfony's Process component to ensure proper isolation.

---

## Requirements
- Drupal 11+
- Python 3 installed and accessible via `python3`
- `symfony/process` component (comes with Drupal core)

---

## Maintainers
This module is actively maintained. Contributions, suggestions, and issues are welcome via the project issue queue.

---

## Credits
Inspired by the need to bridge modern data workflows with robust CMS architectures. Thanks to the Drupal and Python communities for their open-source brilliance.

